
/** INCLUDES *******************************************************/
#include <timers.h>
#include <pwm.h>
#include <adc.h>

#include "user.h"

/** V A R I A B L E S ********************************************************/
#pragma udata
volatile int tmr0_cycle_flag = 0;


/** D E C L A R A T I O N S **************************************************/
#pragma code

void User_Init(void)
  {
    int value;
    
    TRIS_OUTPUT;
    LED_ON;
            
    // Initialize voltage sensor
    OpenADC( 
        ADC_FOSC_64 // Book page 98, max value for 20 MHz
        & ADC_RIGHT_JUST //Results in least significant bits
        & ADC_20_TAD, // collection time interval
        ADC_CH0 & ADC_CH1 & ADC_CH2 // CH0=AN0 (pin 2), CH1=AN1...
        & ADC_INT_OFF // not interrupts (use polling)
        & ADC_VREFPLUS_VDD // use +5 for reference voltage
        & ADC_VREFMINUS_VSS, // use 0 for reference
        12 // bits 0 through 3 of the ADCON1 register
      );
     
    // The Timer0 clock interrupt forms our clock.
    OpenTimer0( 
        TIMER_INT_ON & 
        T0_SOURCE_INT &
        T0_16BIT &
        T0_PS_1_64 
      );
      
    // INTCON=0;    //make sure interrupts are disable
    // INTCONbits.GIE=1;  //enable global interrupts
    // INTCONbits.PEIE=1;  //enable peripheral interrupts
    // INTCONbits.TMR0IE=1; //enable TMR0 overflow interrupt enable bit
    INTCON=0xe0;   //all 4 lines above combined     
    
    
    // Timer1 counts the pulses from the 144 circuit.
    // 4kHz signal should fit well with a prescale of 1
    OpenTimer1( 
        TIMER_INT_OFF & 
        T1_16BIT_RW &      // max count of 65535    
        T1_SOURCE_EXT &  
        T1_PS_1_1 &   // options are 1,2,4,8
        T1_OSC1EN_OFF &
        T1_SYNC_EXT_OFF 
      ); 
    
    // Timer2 works with the PWM system.
    OpenTimer2( 
        TIMER_INT_OFF & 
        TMR2_PRE_SCALE
      ); 

    OpenPWM1( PWM_PR2 ); // 0 to 255
    SetDCPWM1( PWM_DC ); //0 to 1023
    
    TRISDbits.TRISD2=0;
  }

#define V_DC_NEW    1        
#define V_DC_BUSY   2
#define V_DC_READ   3
#define V_DISABLED  7



void display_rpm()
  {
    unsigned int value;
    unsigned long osc_count;
    unsigned long t0_delta;
    unsigned long t0_duration;
    unsigned long seconds;
    unsigned long rpm;

    osc_count = ReadTimer1();    
    t0_delta = ReadTimer0(); // number of ticks since last overflow.
    
    WriteTimer1(0);
    WriteTimer0(0);
    
    
    t0_duration = 65535L + t0_delta; // we assume only one overflow.
    
    // Seconds = t0_duration/((Hz/4)/PS)

    // Hz = 48 MHz
    // PS = 64
    // seconds = t0_duration/((48,000,000/4)/64)
    //         = t0_duration / (12,000,000/64)
    //         = t0_duration / 187500

    // We want to find RPM
    // pulses/second = osc_count/seconds
    //               = osc_count/(t0_duration/187500)
    //               = osc_count*187500/t0_duration
    
    
    // pulses/minute = 60 * pulses/second
    //               = 60 * osc_count*187500/t0_duration
    
    // RPM           = (pulses/minute)/2
    //               = 30 * osc_count*187500/t0_duration
    //               = 30 *187500 * osc_count / t0_duration
    //               = 5625000 * osc_count / t0_duration

/*
    rpm = osc_count;
    rpm = (INV_TIME_INTERVAL_MS*3L*rpm)/100L; //INV_TIME_INTERVAL_MS*30L*rpm/1000L;
    value = rpm/500;
*/

    rpm = 5625000L * osc_count / t0_duration;
    value = 1+rpm/500L;
    
    LED_OFF;
                
    switch (value)
      {
        default: 
        case 14: LED_P14 = 1;
        case 13: LED_P13 = 1;
        case 12: LED_P12 = 1;
        case 11: LED_P11 = 1;
        case 10: LED_P10 = 1;
        case 9: LED_P9 = 1;
        case 8: LED_P8 = 1;
        case 7: LED_P7 = 1;
        case 6: LED_P6 = 1;
        case 5: LED_P5 = 1;
        case 4: LED_P4 = 1;
        case 3: LED_P3 = 1;
        case 2: LED_P2 = 1;
        case 1: LED_P1 = 1;
        case 0: break; // Add a LED_P0

      }
  }
  
void User_Main(void)
  {       
    static int my_debug = 0;
    static unsigned int osc_count=0;
    static int adc_state = V_DC_NEW;
    static int volt_dc = 0;
    static int dc = 0;
    int diff = 0;
                

    /* Check timer
     * At regular intervals, the tmr0_cycle_flag will be set to 1
     */
    if (tmr0_cycle_flag)
      {
        tmr0_cycle_flag = 0;
        display_rpm();
      }
      
    
    // measure voltage
    switch (adc_state)
      {
        case V_DISABLED: break;
        
        /* VOLT12A */
        case V_DC_NEW:
            SetChanADC( ADC_CH0 );
            ConvertADC(); 
            adc_state = V_DC_BUSY;            
          break;
          
        case V_DC_BUSY:
            if (! BusyADC())
              adc_state = V_DC_READ; 
          break;
          
        case V_DC_READ:
            volt_dc = ReadADC();
            adc_state = V_DC_NEW; 
            
            volt_dc = VOLT_TO_DC(volt_dc);
            
            if (volt_dc > dc)
              diff = volt_dc - dc;
            else
              diff = dc - volt_dc;
            
            if (diff > 2)
              {
                dc = volt_dc;
                SetDCPWM1( dc );
              }
          break; 
      }

    
}//end UserMain

